__name__ = "deltas"
__version__ = "0.6.0"
__author__ = "Aaron Halfaker"
__author_email__ = "aaron.halfaker@gmail.com"
__description__ = "An experimental diff library for generating " + \
                  "operation deltas that represent the " + \
                  "difference between two sequences of comparable items."
__license__ = "MIT"
__url__ = "https://github.com/halfak/deltas"

all = [__name__, __version__, __author__, __author_email__, __description__,
       __license__, __url__]
